package be.iminds.ilabt.jfed.rspec.statistics;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.AdvertisementCounts;
import javafx.util.Pair;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.lang.reflect.Array;
import java.util.*;

public class RSpecStatistics implements AdvertisementCounts {
    private boolean countWarnings = false;
    private boolean countErrors = false;

    private final int count;
    private final int countRaw;
    private final int countVm;
    private final int countIpv4;
    private final int countOpenflow;

    private final int countTotal;
    private final int countRawTotal;
    private final int countVmTotal;
    private final int countIpv4Total;
    private final int countOpenflowTotal;

    private final int skippedCount;

    @Nullable private final Date date;

    @Nonnull private final Set<String> explicitSkippedHwTypes;
    @Nonnull private final Set<String> implicitSkippedHwTypes;
    @Nonnull private final Set<String> countedHwTypes;

    //in Pair: first avail, then all
    @Nonnull private final Map<String, CountPair> countPerHwType;

    public static class CountPair {
        private final int total;
        private final int available;

        public CountPair(int total, int available) {
            this.total = total;
            this.available = available;
        }

        public int getTotal() {
            return total;
        }

        public int getAvailable() {
            return available;
        }

        public CountPair newWithIncTotal(int extraTotal) {
            return new CountPair(total+extraTotal, available);
        }
        public CountPair newWithIncAvailable(int extraAvailable) {
            return new CountPair(total, available+extraAvailable);
        }
        public CountPair newWithInc(Integer extraTotal, int extraAvailable) {
            return new CountPair(total+extraTotal, available+extraAvailable);
        }
    }

    public RSpecStatistics(boolean countWarnings, boolean countErrors, int count, int countTotal, int countRaw, 
                           int countIpv4, int countOpenflow, int countVm, int countRawTotal, int countVmTotal, 
                           int countIpv4Total, int countOpenflowTotal, int skippedCount,
                           @Nullable Date date,
                           @Nonnull Set<String> explicitSkippedHwTypes,
                           @Nonnull Set<String> implicitSkippedHwTypes,
                           @Nonnull Set<String> countedHwTypes,
                           @Nonnull Map<String, CountPair> countPerHwType) {
        this.countWarnings = countWarnings;
        this.countErrors = countErrors;
        this.count = count;
        this.countTotal = countTotal;
        this.countRaw = countRaw;
        this.countIpv4 = countIpv4;
        this.countOpenflow = countOpenflow;
        this.countVm = countVm;
        this.countRawTotal = countRawTotal;
        this.countVmTotal = countVmTotal;
        this.countIpv4Total = countIpv4Total;
        this.countOpenflowTotal = countOpenflowTotal;
        this.skippedCount = skippedCount;
        this.date = date;
        this.explicitSkippedHwTypes = explicitSkippedHwTypes;
        this.implicitSkippedHwTypes = implicitSkippedHwTypes;
        this.countedHwTypes = countedHwTypes;
        this.countPerHwType = countPerHwType;
    }

    @Nullable
    @Override
    public Integer getCount(@Nonnull CountedObject countedObject, @Nonnull CountScope countScope) {
        assert countScope == CountScope.ALL || countScope == CountScope.AVAILABLE;
        switch (countedObject) {
            case ANY: return countScope == CountScope.ALL ? countTotal : count;
            case IPV4: return countScope == CountScope.ALL ? countIpv4Total : countIpv4;
            case OPENFLOW: return countScope == CountScope.ALL ? countOpenflowTotal : countOpenflow;
            case RAW_PC: return countScope == CountScope.ALL ? countRawTotal : countRaw;
            case VM: return countScope == CountScope.ALL ? countVmTotal : countVm;
            default: throw new IllegalArgumentException("Unsupported countedObject="+countedObject);
        }
    }

    @Nullable
    @Override
    public Integer getHwCount(@Nonnull String s, @Nonnull CountScope countScope) {
        CountPair p = countPerHwType.get(s);
        if (p == null) return null;
        switch (countScope) {
            case ALL: return p.getTotal();
            case AVAILABLE: return p.getAvailable();
            default: throw new IllegalArgumentException("Unsupported CountScope: "+countScope);
        }
    }

    @Nonnull
    @Override
    public Collection<AdvertisementCountEntry> getCountEntries() {
        return Arrays.asList(
                new AdvertisementCountEntry(CountedObject.ANY, CountScope.ALL, countTotal),
                new AdvertisementCountEntry(CountedObject.ANY, CountScope.AVAILABLE, count),
                new AdvertisementCountEntry(CountedObject.IPV4, CountScope.ALL, countIpv4Total),
                new AdvertisementCountEntry(CountedObject.IPV4, CountScope.AVAILABLE, countIpv4),
                new AdvertisementCountEntry(CountedObject.OPENFLOW, CountScope.ALL, countOpenflowTotal),
                new AdvertisementCountEntry(CountedObject.OPENFLOW, CountScope.AVAILABLE, countOpenflow),
                new AdvertisementCountEntry(CountedObject.RAW_PC, CountScope.ALL, countRawTotal),
                new AdvertisementCountEntry(CountedObject.RAW_PC, CountScope.AVAILABLE, countRaw),
                new AdvertisementCountEntry(CountedObject.VM, CountScope.ALL, countVmTotal),
                new AdvertisementCountEntry(CountedObject.VM, CountScope.AVAILABLE, countVm)
        );
    }

    @Override
    public int getCountEntriesSize() {
        return CountedObject.values().length * 2;
    }

    @Nonnull
    @Override
    public Collection<AdvertisementHwCountEntry> getHwCountEntries() {
        List<AdvertisementHwCountEntry> res = new ArrayList<>();
        for (Map.Entry<String, CountPair> e : countPerHwType.entrySet()) {
            assert e.getKey() != null;
            assert e.getValue() != null;
            res.add(new AdvertisementHwCountEntry(e.getKey(), CountScope.ALL, e.getValue().getTotal()));
            res.add(new AdvertisementHwCountEntry(e.getKey(), CountScope.AVAILABLE, e.getValue().getAvailable()));
        }
        return res;
    }

    @Override
    public int getHwCountEntriesSize() {
        return countPerHwType.size();
    }

    @Override
    public List<String> getHwCountKeys() {
        return new ArrayList<>(countPerHwType.keySet());
    }

    @Nullable
    @Override
    public Date getCountDate() {
        return date;
    }


    public boolean isCountWarnings() {
        return countWarnings;
    }

    public boolean isCountErrors() {
        return countErrors;
    }

    public int getCount() {
        return count;
    }

    public int getCountRaw() {
        return countRaw;
    }

    public int getCountVm() {
        return countVm;
    }

    public int getCountIpv4() {
        return countIpv4;
    }

    public int getCountOpenflow() {
        return countOpenflow;
    }

    public int getCountTotal() {
        return countTotal;
    }

    public int getCountRawTotal() {
        return countRawTotal;
    }

    public int getCountVmTotal() {
        return countVmTotal;
    }

    public int getCountIpv4Total() {
        return countIpv4Total;
    }

    public int getCountOpenflowTotal() {
        return countOpenflowTotal;
    }

    public int getSkippedCount() {
        return skippedCount;
    }

    @Nullable
    public Date getDate() {
        return date;
    }

    @Nonnull
    public Set<String> getExplicitSkippedHwTypes() {
        return explicitSkippedHwTypes;
    }

    @Nonnull
    public Set<String> getImplicitSkippedHwTypes() {
        return implicitSkippedHwTypes;
    }

    @Nonnull
    public Set<String> getCountedHwTypes() {
        return countedHwTypes;
    }

    @Nonnull
    public Map<String, CountPair> getCountPerHwType() {
        return countPerHwType;
    }
}
